@@ -540,9 +540,9 @@ g_tls_certificate_gnutls_convert_flags (guint gnutls_flags)
   return gtls_flags;
 }
 
-GTlsCertificateFlags
-g_tls_certificate_gnutls_verify_identity (GTlsCertificateGnutls *gnutls,
-					  GSocketConnectable    *identity)
+static gboolean
+verify_identity_hostname (GTlsCertificateGnutls *gnutls,
+			  GSocketConnectable    *identity)
 {
   const char *hostname;
 
@@ -551,14 +551,72 @@ g_tls_certificate_gnutls_verify_identity (GTlsCertificateGnutls *gnutls,
   else if (G_IS_NETWORK_SERVICE (identity))
     hostname = g_network_service_get_domain (G_NETWORK_SERVICE (identity));
   else
-    hostname = NULL;
+    return FALSE;
 
-  if (hostname)
+  return gnutls_x509_crt_check_hostname (gnutls->priv->cert, hostname);
+}
+
+static gboolean
+verify_identity_ip (GTlsCertificateGnutls *gnutls,
+		    GSocketConnectable    *identity)
+{
+  GInetAddress *addr;
+  int i, ret = 0;
+  gsize addr_size;
+  const guint8 *addr_bytes;
+
+  if (G_IS_INET_SOCKET_ADDRESS (identity))
+    addr = g_object_ref (g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (identity)));
+  else {
+    const char *hostname;
+
+    if (G_IS_NETWORK_ADDRESS (identity))
+      hostname = g_network_address_get_hostname (G_NETWORK_ADDRESS (identity));
+    else if (G_IS_NETWORK_SERVICE (identity))
+      hostname = g_network_service_get_domain (G_NETWORK_SERVICE (identity));
+    else
+      return FALSE;
+
+    addr = g_inet_address_new_from_string (hostname);
+    if (!addr)
+      return FALSE;
+  }
+
+  addr_bytes = g_inet_address_to_bytes (addr);
+  addr_size = g_inet_address_get_native_size (addr);
+
+  for (i = 0; ret >= 0; i++)
     {
-      if (gnutls_x509_crt_check_hostname (gnutls->priv->cert, hostname))
-	return 0;
+      char san[500];
+      size_t san_size;
+
+      san_size = sizeof (san);
+      ret = gnutls_x509_crt_get_subject_alt_name (gnutls->priv->cert, i,
+						  san, &san_size, NULL);
+
+      if ((ret == GNUTLS_SAN_IPADDRESS) && (addr_size == san_size))
+	{
+	  if (memcmp (addr_bytes, san, addr_size) == 0)
+	    {
+	      g_object_unref (addr);
+	      return TRUE;
+	    }
+	}
     }
 
+  g_object_unref (addr);
+  return FALSE;
+}
+
+GTlsCertificateFlags
+g_tls_certificate_gnutls_verify_identity (GTlsCertificateGnutls *gnutls,
+					  GSocketConnectable    *identity)
+{
+  if (verify_identity_hostname (gnutls, identity))
+    return 0;
+  else if (verify_identity_ip (gnutls, identity))
+    return 0;
+
   /* FIXME: check sRVName and uniformResourceIdentifier
    * subjectAltNames, if appropriate for @identity.
    */
